/**
* DS3231.cpp
* Leeman Geophysical LLC (c) 2018
* DS3231 RTC Driver
* 
* Work with the DS3231 RTC.
*/

#include "DS3231.h"
#include "simpletools.h"
#include <propeller.h>
#include "pins.h"

i2c *eeBus;

uint8_t bcd_to_dec(uint8_t val)
{
  return ( (val/16*10) + (val%16) );
}

uint8_t dec_to_bcd(uint8_t val)
{
  return (((val / 10) << 4) + (val % 10) );
}

void DS3231::start(int address, char data_pin, char clk_pin, char sq_pin) {
  eeBus = i2c_newbus(clk_pin, data_pin, 0);
  dev_address = address;
  square_pin = sq_pin;  
}  


void DS3231::ocsillator_enable(char enable) {
  /*
   * 0 - oscillator is started
   * 1 - oscillator is stopped when on Vbat
   * Is set to 0 on power up. Oscillator is always on when powered by
   * Vcc. When disabled, all register data are static. Value is stored
   * in bit 7 of the control register.
   */
  change_byte_register(REG_CONTROL, 0b10000000, enable << 7); 
}  

void DS3231::square_wave_enable(char enable) {
  /*
   * 1 with INTCN=0 and Vcc<Vpf, square wave is enabled
   * 0 the pin goes high impedance when Vcc<Vpf
   * Set to 0 on power up
   */
  change_byte_register(REG_CONTROL, 0b01000000, enable << 6); 
}  

void DS3231::convert_temperature_enable(char enable) {
  /*
   * 1 - Forces the temperature sensor to convert the temperature into a
   * digital coe and execute to TCXO alorithm to update the capacitance
   * array to the oscillator. This can only happen when a conversion is not
   * already in progress. The user should check the status bit BSY before
   * forcing the controller to start a new TCXO execution. A user-initiated
   * temperature conversion does not affect the internal 64-second update cycle.
   * A user-initiated temperature conversion does not affect the BSY bit for
   * approximately 2ms. The CONV bit remains at a 1 from the time it is written
   * until the conversion is finished, at which time both CONV and BSY go to 0.
   * The CONV bit should be used when monitoring the status of a user-initiated
   * conversion.
   */
   change_byte_register(REG_CONTROL, 0b00100000, enable << 5);
}

void DS3231::square_wave_frequency(char rate) {
  /*
   * Set the square wave output frequency when enabled.
   * 0 - 1 Hz
   * 1 - 1.024 kHz
   * 2 - 4.096 kHz
   * 3 - 8.192 kHz
   */
  change_byte_register(REG_CONTROL, 0b00011000, rate << 3);
}  

void DS3231::interrupt_control(char enable){
  /*
   * 0 - square wave output is enabled
   * 1 - square wave output is disabled (alarm outputs enabled)
   * Set to 1 on power up
   */
  change_byte_register(REG_CONTROL, 0b0000100, enable << 2);
}  

void DS3231::alarm_two_enable(char enable){
  /*
   * 0 - alarm 2 does not initiate an interrupt signal
   * 1 - alarm 2 does initiate an interrupt signal
   * Disabled on power up
   */
  change_byte_register(REG_CONTROL, 0b00000010, enable << 1);
}  

void DS3231::alarm_one_enable(char enable){
  /*
   * 0 - alarm 1 does not initiate an interrupt signal
   * 1 - alarm 1 does initiate an interrupt signal
   * Disabled on power up
   */
  change_byte_register(REG_CONTROL, 0b00000001, enable);
} 

char DS3231::get_oscillator_stop_flag(void) {
  /*
   * A logic 1 in this bit indicates that the oscillator either is stopped
   * or was stopped for some period and may be used to judge the validity of
   * the timekeeping data. This bit is set to logic 1 any time that the
   * oscillator stops. The following are examples of conditions that cna cause
   * the OSF bit to be set.
   * 1. The first time power is applied
   * 2. The voltages present on both Vcc and Vbat are insufficient to support oscillation
   * 3. The ~EOSC bit is turned off in battery-backed mode
   * 4. External influences of the crystal
   * This bit remains at logic 1 until written to logic 0
   */
  unsigned char bus_data;
  i2c_in(eeBus, dev_address, REG_STATUS, 1, &bus_data, 1);
  bus_data = bus_data & 0b10000000;
  return bus_data >> 7;
}
  
void DS3231::clear_oscillator_stop_flag(void) {
  /*
   * Clear the oscillator stop flag.
   */
  change_byte_register(REG_STATUS, 0b10000000, 0);
}
  
void DS3231::clock_output_enable(char enable) {
  /*
   * 1 - The 32kHz pin is enabled and outputs a 32.768 kHz square wave
   * 0 - the 32kHz pin is in a high-impedance state
   * Enabled on power up.
   */
  change_byte_register(REG_STATUS, 0b00001000, enable << 3);
}  

char DS3231::get_alarm_one_flag(void) {
  /*
   * Check status of alarm 1.
   */
  unsigned char bus_data;
  i2c_in(eeBus, dev_address, REG_STATUS, 1, &bus_data, 1);
  bus_data = bus_data & 0b00000001;
  return bus_data;
}  

char DS3231::get_alarm_two_flag(void) {
  /*
   * Check status of alarm 2.
   */
  unsigned char bus_data;
  i2c_in(eeBus, dev_address, REG_STATUS, 1, &bus_data, 1);
  bus_data = bus_data & 0b00000010;
  return bus_data >> 1;
}  

void DS3231::set_aging_offset(char offset) {
  /*
   * Set the aging offset.
   */
  change_byte_register(REG_AGING_OFFSET, 0b11111111, offset);
}  

int DS3231::get_temperature(void) {
  // TODO: Implement
  return 0;
}  

void DS3231::set_time(char year, char month, char date, char hour, char minute, char second){
  /*
   * Set the RTC time.
   */
  
  // Set seconds
  change_byte_register(REG_SECONDS, 0b01111111, dec_to_bcd(second));
  
  // Set minutes
  change_byte_register(REG_MINUTES, 0b01111111, dec_to_bcd(minute));
  
  // Set hours mode to 24 hours
  change_byte_register(REG_HOURS, 0b01000000, 0);
  
  // Set hours
  change_byte_register(REG_HOURS, 0b00111111, dec_to_bcd(hour));
  
  // Set Century TODO
  
  // Set months
  change_byte_register(REG_MONTH_CENTURY, 0b00011111, dec_to_bcd(month));
  
  // Set days
  change_byte_register(REG_DATE, 0b00111111, dec_to_bcd(date));
  
  // Set years
  change_byte_register(REG_YEARS, 0b11111111, dec_to_bcd(year));
  
}  

void DS3231::read_time() {
  unsigned char bus_data;
  
  i2c_in(eeBus, dev_address, REG_SECONDS, 1, &bus_data, 1);
  seconds = ((bus_data & 0b01110000) >> 4) * 10;
  seconds = seconds + (bus_data & 0b00001111);
  
  i2c_in(eeBus, dev_address, REG_MINUTES, 1, &bus_data, 1);
  minutes = ((bus_data & 0b01110000) >> 4) * 10;
  minutes = minutes + (bus_data & 0b00001111); 
  
  i2c_in(eeBus, dev_address, REG_HOURS, 1, &bus_data, 1);
  hours = ((bus_data & 0b00110000) >> 4) * 10;
  hours = hours + (bus_data & 0b00001111);
  
  i2c_in(eeBus, dev_address, REG_DATE, 1, &bus_data, 1);
  date = ((bus_data & 0b00110000) >> 4) * 10;
  date = date + (bus_data & 0b00001111);
   
  i2c_in(eeBus, dev_address, REG_MONTH_CENTURY, 1, &bus_data, 1);
  month = ((bus_data & 0b00010000) >> 4) * 10;
  month = month + (bus_data & 0b00001111);
  
  // TODO: Need to fix century stuff here
  i2c_in(eeBus, dev_address, REG_YEARS, 1, &bus_data, 1);
  year = ((bus_data & 0b11110000) >> 4) * 10;
  year = year + (bus_data & 0b00001111);
}  

void DS3231::change_byte_register(int reg, char mask, char value){
  unsigned char bus_data;
  bus_data = i2c_in(eeBus, dev_address, reg, 1, &bus_data, 1);
  bus_data = bus_data & ~mask;
  bus_data = bus_data | value;
  i2c_out(eeBus, dev_address, reg, 1, &bus_data, 1); 
  while(i2c_busy(eeBus, dev_address)); 
}  
