/**
* TML.cpp
* Leeman Geophysical LLC (c) 2018
* Tilt Meter Logger
* 
* Control logging and other functions of the TML associated with the PTM100.
*/


/*
 * TODO
 * - Fix SD unmount/remount issue
 * - Set file time on SD card files
 */

#include <stdio.h>
#include <propeller.h>
#include "pins.h"
#include "simpletools.h"
#include <fdserial.h>
#include "logging.h"
#include "TML.h"  
  
front_panel panel;

static int sd_cog;
char time_match_interval = 0;
char time_alarm_handled = 0;
int packets_written = 0;
const int n_readings = 10;
char last_minute_written = 99;


void (*do_state)(void);


// Uncomment the next line to send debug messages to comp_serial
//#define ENABLE_DEBUG

int main(){
  /*
   * Main state machine loop
   * - Do the initialize state first, then the state machine will be
   *   self directing by setting the do_state function pointer.
   */
  do_state = handle_state_initialize;
  while(1) {
    do_state();
  }    
}


/*
 * State handlers
 */

void handle_state_initialize(void) {
  /*
   * State: Initialize
   * Tasks:
   * - Start computer serial port
   * - Launch RTC time keeping cog
   * Next State: Update Panel
   */
  sysstatus.clear();
  comp_serial = fdserial_open(PIN_COMP_RX, PIN_COMP_TX, 0, 115200);
  send_debug("STATE: Initialize\n");
  
  // Start the software/hardware RTC Cog
  rtc_cog = cogstart(&rtc_runner, NULL, rtcstack, sizeof(rtcstack));
  tiltmeter.on();
  // Clear out PTM buffer
  tiltmeter.clear_buffer();
  rx_cog = cogstart(&receive_runner, NULL, rxstack, sizeof(rxstack));
  
  // Next State
  do_state = handle_state_update_panel;
}  


void handle_state_update_panel(void) {
  /*
   * State: Update Panel
   * Tasks:
   * - Read front panel switches and update panel state attributes
   * - Update panel lights
   * Next State: write_data (instrument mode) OR interval_time_check (timed mode)
   */
  send_debug("STATE: Update Panel\n");
  panel.error_status = sysstatus.is_error();
  panel.update_switches();
  panel.update_lights();
  panel.handle_serial();

 
  // Determine where to go next
  switch (panel.interval_status) {
    case 0: // Instrument control mode
      do_state = handle_state_write_data;
      break;
    default: // Any timed mode
      do_state = handle_state_near_time_check;
      break;
  }  // switch interval status            
}  // handle_state_update_panel


void handle_state_write_data(void) {
  /*
   * State: Write Buffer
   * Tasks:
   * - Make sure the SD card is mounted and available
   * - Get the filename for the data packet
   * - Write serial buffer from instrument to SD Card
   * Next State: Update Panel
   */
  send_debug("STATE: Write Data\n");
  
  if (panel.logging_status == 0){
    do_state = handle_state_update_panel;
    return;
  }    
  
  if (data_ring.data_length() > 0) {  // Data are there to be written
    // Light writing light if panel is enabled
    if (panel.light_status == 1) {high(PIN_WRITING_LED);}    
  
    ensure_sdcard_mount(panel.logging_status, sd_mounted);
  
    char fname[13];
    //fname = "TEMPTEMP.TXT";
    //FILE* fp = fopen(fname, "a");
    FILE* fp = fopen("TEMP.TXT", "a");
    //get_filename(fname, tiltmeter.packet_dt);
    
    ptmpacket p = data_ring.read();
    
    char log_buffer[225];
    char log_buffer_len = sprintf(log_buffer, "%02d/%02d/%02dT%02d:%02d:%02d.%03d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\r\n",
          p.packet_dt.year, p.packet_dt.month, p.packet_dt.date,
          p.packet_dt.hour, p.packet_dt.minute, p.packet_dt.second,
          p.packet_dt.millisecond, p.data_valid_cnt, p.x_tilt_high,
          p.x_tilt_low, p.y_tilt_high, p.y_tilt_low, p.x_tilt, p.y_tilt,
          p.board_temperature, p.mag_temperature, p.metal_temperature,
          p.mag_x, p.mag_y, p.mag_z, p.acc_x, p.acc_y, p.acc_z);   
    fwrite(log_buffer, 1, log_buffer_len, fp);
    
    fclose(fp);  
    low(PIN_WRITING_LED);
    //packets_written++;  
  } // if buffer items > 0       
   
  do_state = handle_state_update_panel;
}  


void handle_state_near_time_check(void){
  /*
   * State: Near Time Check
   * Tasks:
   * - See if it is time to wakeup and write (timed mode!)
   * Next State:
   * - Update Panel (not time to wakeup)
   * - Get N Readings (time to wakeup)
   */

  // Check if the time remainder is zero, if so, it matches, if not
  // it does not and reset the alarm handled bit.
  send_debug("STATE: Near Time Check\n");
  
  // See if the trigger condition is met - will be true for 1 minute
  char trigger_met = 0;
  if (rtcdatetime.minute % panel.interval_status == 0) {
    trigger_met = 1;
  }
  else {
    trigger_met = 0;
    packets_written = 0;
    
  }        
      
  // See if we have already written data this minute
  char already_done = 0;
  if (rtcdatetime.minute == last_minute_written) {
    already_done = 1;
    packets_written = 0;
  }
  
  // Make sure the instrument is off if not needed (front panel switch bounce)
  if(!trigger_met || already_done) {
    tiltmeter.off();
  }    
  
  // Determine next state
  if (trigger_met && !already_done) {  // Time and we haven't already taken care of it
    do_state = handle_state_get_n_readings;
  }
  else {
    do_state = handle_state_update_panel;
  }           
}  

void handle_state_get_n_readings(void){
  /*
   * State: Get N Readings
   * Tasks:
   * - Turn on instrument
   * - Get a write n readings to card
   * - Turn off instrument
   * Next State:
   * - Update Panel
   */
  send_debug("STATE: Get N Readings\n");
  tiltmeter.on();
   
  // If we dont' have enough data written yet, we go do that
  if (packets_written < n_readings) {
    do_state = handle_state_update_panel; //TEMP was check serial
  }    
  else {  // We have enough, shut it down
    tiltmeter.off();
    time_alarm_handled = 1; // Flag that we've finished with time alarm
    last_minute_written = rtcdatetime.minute;
    do_state = handle_state_update_panel;
  }  
}  

/*
 * Helpers
 */

void echo_packet(void) {
  if (panel.light_status == 1) {
    for (char i=0; i < tiltmeter.inst_ser_buffer_position; i++) {
      fdserial_txChar(comp_serial, tiltmeter.inst_ser_buffer[i]);
    }  // for
  }  // if  
}    

void get_filename(char *buffer, datetime dt) {
  /*
   * Determine the filename to write to for the date/time.
   */
  sprintf(buffer, "%02d%02d%02dTM.txt", dt.year, dt.month, dt.date);
}  


int get_milliseconds(void) {
  return (CNT - rtc.valid_cnt) / (CLKFREQ / 1000);
}  // get_milliseconds  


void receive_runner(void *par) {
  /*
   * Run a cog to receive serial data and put complete packets into a ring buffer.
   */

  while(1) {
    // Receive data into local buffer until it is a verified packet
    tiltmeter.read_new_data();
    echo_packet(); // Only echos if panel switch is on         
  }  // while(1) 
}  // receiver_runner


void rtc_runner(void *par) {
  /*
   * RTC runner runs the RTC in its own cog to provide time stamps.
   * This updates the RTC object and globals.
   */
  rtc.start(0x68, PIN_SDA, PIN_SCL, PIN_PPS);
  rtc.interrupt_control(0);  // Enable square wave output
  rtc.square_wave_frequency(0); // 1 Hz
  
  while(1) {
    // If the do_rtc_set flag is set, we set the clock to rtcsettime
    if (do_rtc_set) {
      rtc.set_time(rtcsettime.year - 2000, rtcsettime.month, rtcsettime.date,
                   rtcsettime.hour, rtcsettime.minute, rtcsettime.second);
      do_rtc_set = 0;
      //dprint(comp_serial, "Set time cog: %d/%d/%d", rtcsettime.year, rtcsettime.month, rtcsettime.date);
    }  // if do_rtc_set                   
    
    waitpeq(1, 1 << PIN_PPS);  // Wait for PPS to be high (middle of second)
    pause(100);  // Pause to get past ratty square wave corners
    waitpeq(0, 1 << PIN_PPS);  // Wait for PPS to be low (on the second)
    rtc.valid_cnt = CNT;
    
    rtc.read_time();  // Update the time
    rtcdatetime.year = rtc.year;
    rtcdatetime.month = rtc.month;
    rtcdatetime.date = rtc.date;
    rtcdatetime.hour = rtc.hours;
    rtcdatetime.minute = rtc.minutes;
    rtcdatetime.second = rtc.seconds;
    rtcdatetime.valid_cnt = rtc.valid_cnt;
  }  //while(1) 
}  //rtc_runner


/*
 * Debug Output
 */
void send_debug(char *msg) {
  #ifdef ENABLE_DEBUG
  writeStr(comp_serial, msg);
  #endif
}  


/*
 * Core Library Overrides
 */
