/**
* ptm100.cpp
* Leeman Geophysical LLC (c) 2018
* Tilt Meter Logger
* 
* Tilt meter interface functionality.
*/

#include <simpletools.h>
#include "pins.h"
#include "ptm100.h"
#include "cbuffer.h"
#include <fdserial.h>

fdserial *inst_serial;
extern datetime rtcdatetime;
extern fdserial *comp_serial;
extern cbuffer data_ring;

void ptm100::on(void) {
  /*
   * Turn on the instrument and connect to the serial port.
   */
  // Should/could this be a set to high impedance?
  if (power_state == 0) {  // Protect trying to allocate already allocated resources
    high(PIN_INST_PWR);
    pause(100);
    inst_serial = fdserial_open(PIN_INST_RX, PIN_INST_TX, 0, 57600);
    power_state = 1;
  }    
}

void ptm100::off(void) {
  /*
   * Turn off the instrument and release the serial port.
   */
  if (power_state == 1) { // Protect trying to deallocate already deallocated resources
    fdserial_close(inst_serial);
    low(PIN_INST_PWR);
    power_state = 0;
  }      
}    


void ptm100::clear_buffer(void){
  inst_ser_buffer_position = 0;  // Reset buffer position
  tabs_detected = 0;  // Reset tab count
  memset(inst_ser_buffer, 0, sizeof inst_ser_buffer);  // Clear out buffer
}    
  
  
void ptm100::read_new_data(void) {
  /*
   * Read new data and decode it. If it's a full sentence, parse and put it in the queue.
   * Currently we expect 15 tabs in a serial sentence for it to be considered valid.
   */
  
  char rx_byte;
  static ptmpacket data_packet;
  
  while(fdserial_rxReady(inst_serial)) {  // Loop while data are in the buffer
    rx_byte = fdserial_rxChar(inst_serial);  // Get the byte in the buffer
    
    // If this is the first character of a new buffer, set a new time
    if (inst_ser_buffer_position == 0) {
      data_packet.packet_dt.millisecond = rtcdatetime.set_millisecond();
      data_packet.packet_dt.year = rtcdatetime.year;
      data_packet.packet_dt.date = rtcdatetime.date;
      data_packet.packet_dt.month = rtcdatetime.month;
      data_packet.packet_dt.hour = rtcdatetime.hour;
      data_packet.packet_dt.minute = rtcdatetime.minute;
      data_packet.packet_dt.second = rtcdatetime.second;
    }
    
    // Add byte to buffer and move position indicator
    inst_ser_buffer[inst_ser_buffer_position] = rx_byte;
    inst_ser_buffer_position++;              
    
    if (rx_byte == 9) {  // If it's a tab, add it to the count
      tabs_detected++;
    }
    
    if (rx_byte == 13) {  // Indicates this is the end of the serial packet
      if (tabs_detected == 15) {  // Full packet 
        sscanf(inst_ser_buffer, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d", 
               &data_packet.data_valid_cnt, &data_packet.x_tilt_high,
               &data_packet.x_tilt_low, &data_packet.y_tilt_high, &data_packet.y_tilt_low,
               &data_packet.x_tilt, &data_packet.y_tilt,
               &data_packet.board_temperature, &data_packet.mag_temperature,
               &data_packet.metal_temperature, &data_packet.mag_x, &data_packet.mag_y, 
               &data_packet.mag_z, &data_packet.acc_x, &data_packet.acc_y, &data_packet.acc_z);
        // Put data packet into ring buffer
        data_ring.write(data_packet);
        clear_buffer();
        break;      
      }
      else {
        // Bad packet, clear it out
        //dprint(comp_serial, "Bad data %d tabs\n", tabs_detected);
        clear_buffer();
        break;         
      } // else
      break;                  
    }  // if rx_byte==13        
  }  // while
}  // func


